// Jerome Greene
// ME608 _ Final Project

#include <Servo.h>
// Create a Servo object
Servo hipServo;

// PIN VARIABLES
const int AIN1 = 13;           // Control pin 1 on the motor driver for the right motor
const int AIN2 = 12;           // Control pin 2 on the motor driver for the right motor
const int PWMA = 11;           // Speed control pin on the motor driver for the right motor
const int BIN1 = 8;            // Control pin 1 on the motor driver for the left motor
const int BIN2 = 9;            // Control pin 2 on the motor driver for the left motor
const int PWMB = 10;           // Speed control pin on the motor driver for the left motor

// 20KG Servo
const int servoPin = 7;

// Distance Sensor_a
int trigPin = 6;
int echoPin = 5;
float distance;
const float stopLimit = 6.0;       // Stop motors when object within 6 inches
// Distance Sensor_b
int trigPinb = 14;
int echoPinb = 15;
float distanceb;

// COLOR SENSOR
#define S0 0
#define S1 1
#define S2 2
#define S3 3
#define OUT 4

#define LedRed    A2
#define LedGreen  A0
#define LedBlue   A1

int R,G,B     = 0;
float Red;
float Blue;
float Green; 


void setup() {
  // Set the motor control pins as outputs
  pinMode(AIN1, OUTPUT);
  pinMode(AIN2, OUTPUT);
  pinMode(PWMA, OUTPUT);
  pinMode(BIN1, OUTPUT);
  pinMode(BIN2, OUTPUT);
  pinMode(PWMB, OUTPUT);

  // Attach the 20KG hip servo
  hipServo.attach(servoPin);

  // Set up distance sensor
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(trigPinb,OUTPUT);
  pinMode(echoPinb,INPUT);

  //Color Sensor
  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  pinMode(OUT, INPUT);
  pinMode(LedRed, OUTPUT);
  pinMode(LedGreen, OUTPUT);
  pinMode(LedBlue, OUTPUT);
  //Setting frequency-scaling to 20%
  digitalWrite(S0,HIGH);
  digitalWrite(S1,LOW);
  
  // Initialize serial communication
  Serial.begin(9600);

  for (int i=0; i<=5; i++){
    digitalWrite(LedRed, !digitalRead(LedRed));
    digitalWrite(LedGreen, !digitalRead(LedGreen));
    digitalWrite(LedBlue, !digitalRead(LedBlue));
    delay(250);
  }
   hipServo.write(0);
    Serial.println("0 degrees");
    delay(1000);  // Wait for servo to reach position
}

void loop() {
  // Get the current distance
  distance = getDistance();

  // Print the distance to the serial monitor
  Serial.print("Distance: ");
  Serial.print(distance);
  Serial.println(" inches");

  Serial.print("Distanceb: ");
  Serial.print(distanceb);
  Serial.println(" inches");

  if (distance > stopLimit) {
    // Drive motors forward at full speed
    driveForward();
    Serial.println("Searching");
  } else if (distance <= stopLimit) {
    // Stop motors when an object is detected within the stop limit
    stopMotors();
    Serial.println("Stopping");
    delay(2000);
    // Move the servo to 45 degrees
    hipServo.write(45);
    Serial.println("45 degrees");
    delay(2000);  // Wait for servo to reach position

    float Red = readAndProcessRed();
    float Green = readAndProcessGreen();
    float Blue = readAndProcessBlue();
    //Detect Object
    if (Red < 15.0, Green < 15.0, Blue < 15.0){
          Serial.println("Salt Detected");
    }
    else if (Red > 25.0){
        Serial.println("Pepper Detected");
    }
    
    // Reverse the left motor and move the right motor forward to turn the robot
    turnAround();
    Serial.println("Robot turned around");// Confirm the action
    delay(2000);  // Time to complete the turn, adjust as necessary
    // Stop motors after turn
    stopMotors();
    Serial.println("Stopped");// Confirm the action
    delay(2000); 
  }

    distanceb = getDistanceb();
    if (distanceb > stopLimit){
      // Drive motors forward at full speed
      driveForward();
      Serial.println("Searching");// Confirm the action
    }
    else if (distanceb <= stopLimit){
      stopMotors();
      Serial.println("Stopped");// Confirm the action
      delay(1000);
      hipServo.write(0);
      Serial.println("0 degrees");
      delay(1000);
      turnAround();
      delay(1000);  // Wait for servo to reach position
      stopMotors();
      Serial.println("Task Completed");
    }
  
delay(500);
}


float getDistance() {
  float echoTime; // Time it takes for a ping to bounce off an object
  float calculatedDistance; // Distance calculated from the echo time

  // Send out an ultrasonic pulse that's 10ms long
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  // Measure the echo time
  echoTime = pulseIn(echoPin, HIGH);

  // Calculate the distance in inches
  calculatedDistance = echoTime / 148.0;

  // Return the calculated distance
  return calculatedDistance;
}
float getDistanceb() {
  float echoTimeb; // Time it takes for a ping to bounce off an object
  float calculatedDistanceb; // Distance calculated from the echo time

  // Send out an ultrasonic pulse that's 10ms long
  digitalWrite(trigPinb, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPinb, LOW);

  // Measure the echo time
  echoTimeb = pulseIn(echoPinb, HIGH);

  // Calculate the distance in inches
  calculatedDistanceb = echoTimeb / 148.0;

  // Return the calculated distance
  return calculatedDistanceb;
}
void driveForward() {
  digitalWrite(AIN1, HIGH);
  digitalWrite(AIN2, LOW);
  analogWrite(PWMA, 255);
  digitalWrite(BIN1, HIGH);
  digitalWrite(BIN2, LOW);
  analogWrite(PWMB, 255);
}

void stopMotors() {
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, LOW);
  analogWrite(PWMA, 0);
  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, LOW);
  analogWrite(PWMB, 0);
}

void turnAround() {
  // Assuming this code correctly turns the robot around
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, HIGH);
  analogWrite(PWMA, 255);
  digitalWrite(BIN1, HIGH);
  digitalWrite(BIN2, LOW);
  analogWrite(PWMB, 255);
  delay(500);  // Adjust this delay based on the time needed to turn around
}

float readAndProcessRed() {
  float R, G, B;  // Variables to store frequency values for Red, Green, Blue

  // Setting red filtered photodiodes to be read
  digitalWrite(S2, LOW);
  digitalWrite(S3, LOW);
  R = pulseIn(OUT, LOW);  // Reading the output Red frequency
  delay(100);

  // Detect colors based on sensor values
  if (R > 20 && R < 35 && G > 80 && G < 105 && B > 70 && B < 90) {  // to detect red
    digitalWrite(LedRed, HIGH);
  } else if (R > 75 && R < 100 && G > 60 && G < 85 && B > 75 && B < 95) {  // to detect green
    digitalWrite(LedGreen, HIGH);
  } else if (R > 95 && R < 115 && G > 70 && G < 95 && B > 30 && B < 55) {  // to detect blue
    digitalWrite(LedBlue, HIGH);
  } else {
    digitalWrite(LedRed, LOW);
// Print RGB Sensor Values
  Serial.print("R= ");
  Serial.print(R);
  Red = R;
delay(1000);
  }
  return (Red);
  }

 float readAndProcessGreen() {
  float R, G, B;  // Variables to store frequency values for Red, Green, Blue

  // Setting green filtered photodiodes to be read
  digitalWrite(S2, HIGH);
  digitalWrite(S3, HIGH);
  G = pulseIn(OUT, LOW);  // Reading the output Green frequency
  delay(100);

  // Detect colors based on sensor values
  if (R > 20 && R < 35 && G > 80 && G < 105 && B > 70 && B < 90) {  // to detect red
    digitalWrite(LedRed, HIGH);
  } else if (R > 75 && R < 100 && G > 60 && G < 85 && B > 75 && B < 95) {  // to detect green
    digitalWrite(LedGreen, HIGH);
  } else if (R > 95 && R < 115 && G > 70 && G < 95 && B > 30 && B < 55) {  // to detect blue
    digitalWrite(LedBlue, HIGH);
  } else {
    digitalWrite(LedRed, LOW);
    digitalWrite(LedGreen, LOW);
    digitalWrite(LedBlue, LOW);
  }

  // Print RGB Sensor Values
  Serial.print("G= ");
  Serial.print(G);
  Green = G;
  
  delay(1000);
  return (Green);
}


float readAndProcessBlue() {
  float R, G, B;  // Variables to store frequency values for Red, Green, Blue

  // Setting blue filtered photodiodes to be read
  digitalWrite(S2, LOW);
  digitalWrite(S3, HIGH);
  B = pulseIn(OUT, LOW);  // Reading the output Blue frequency
  delay(100);

  // Detect colors based on sensor values
  if (R > 20 && R < 35 && G > 80 && G < 105 && B > 70 && B < 90) {  // to detect red
    digitalWrite(LedRed, HIGH);
  } else if (R > 75 && R < 100 && G > 60 && G < 85 && B > 75 && B < 95) {  // to detect green
    digitalWrite(LedGreen, HIGH);
  } else if (R > 95 && R < 115 && G > 70 && G < 95 && B > 30 && B < 55) {  // to detect blue
    digitalWrite(LedBlue, HIGH);
  } else {
    digitalWrite(LedRed, LOW);
    digitalWrite(LedGreen, LOW);
    digitalWrite(LedBlue, LOW);
  }

  Serial.print("B= ");
  Serial.print(B);
  Blue = B;
  Serial.println();
  delay(1000);
  return (Blue);
}